# Key concepts

## package handles
Packages are referenced by a handle of the form `OWNER/NAME`. 

Teams packages include a prefix, `TEAM:OWNER/NAME`.

## READMEs

A `README.md` is recommended at the root of your package. README files support [full markdown syntax via remarkable](https://jonschlinkert.github.io/remarkable/demo/). READMEs are rendered to HTML on the [package landing page](https://quiltdata.com/package/danWebster/sgRNAs).

## Short hashes
Commands that take hashes support "short hashes", up to uniqueness. In practice, 6-8 characters is sufficient to achieve uniqueness.

```sh
quilt install akarve/examples -x 4594b5
# matches hash 4594b58d64dd9c98b79b628370618031c66e80cbbd1db48662be0b7cac36a74e
```

## Requirements file (`quilt.yml`)
```sh
$ quilt install [@FILENAME]
# quilt.yml is the default if @filename is absent
```

Installs a list of packages specified by a YAML file. The YAML file must contain a `packages` node with a list of packages: 

```yaml
  packages:
    - USER/PACKAGE[/SUBPACKAGE][:h[ash]|:t[ag]|:v[ersion]][:HASH|TAG|VERSION]
```

### Example
  ```
  packages:
    - vgauthier/DynamicPopEstimate   # get latest
    - danWebster/sgRNAs:a972d92      # get a specific version via hash
    - akarve/sales:tag:latest        # get a specific version via tag
    - asah/snli:v:1.0                # get a specific version via version
  ```

## Instances, hashes, tags, and versions
* A package _instance_ is a package handle plus a hash. `akarve/sales:fc7f0b` is an instance. Instances are immutable.
* _Hashes_ are automatically generated by Quilt for each package build.
* _Tags_ are human-readable strings associated with a package instance. Tags can be altered to point to different instances of the same package. The most recent build is automatically tagged `"latest"`.
* _Versions_ are human-readable strings associated with a package instance. Unlike tags, versions can only ever point to a single package instance.
